@@ -10,8 +10,10 @@
 #ifndef XTENSOR_MANIPULATION_HPP
 #define XTENSOR_MANIPULATION_HPP
 
+#include "xbuilder.hpp"
 #include "xstrided_view.hpp"
 #include "xutils.hpp"
+#include "xtensor_config.hpp"
 
 namespace xt
 {
@@ -46,6 +48,42 @@ namespace xt
     template <class E, class S, class Tag = check_policy::none, std::enable_if_t<!std::is_integral<S>::value, int> = 0>
     auto squeeze(E&& e, S&& axis, Tag check_policy = Tag());
 
+    template <class E>
+    auto expand_dims(E&& e, std::size_t axis);
+
+    template <class E>
+    auto atleast_Nd(E&& e);
+
+    template <class E>
+    auto atleast_1d(E&& e);
+
+    template <class E>
+    auto atleast_2d(E&& e);
+
+    template <class E>
+    auto atleast_3d(E&& e);
+
+    template <class E>
+    auto split(E& e, std::size_t n, std::size_t axis = 0);
+
+    template <class E>
+    auto hsplit(E& e, std::size_t n);
+
+    template <class E>
+    auto vsplit(E& e, std::size_t n);
+
+    template <class E>
+    auto flip(E&& e, std::size_t axis);
+
+    template <std::ptrdiff_t N = 1, class E>
+    auto rot90(E&& e, const std::array<std::ptrdiff_t, 2>& axes = {0, 1});
+
+    template<class E>
+    auto roll(E&& e, std::ptrdiff_t shift);
+
+    template<class E>
+    auto roll(E&& e, std::ptrdiff_t shift, std::ptrdiff_t axis);
+
     /****************************
      * transpose implementation *
      ****************************/
@@ -70,7 +108,7 @@ namespace xt
         {
             if (l != layout_type::row_major && l != layout_type::column_major)
             {
-                throw transpose_error("cannot compute transposed layout of dynamic layout");
+                XTENSOR_THROW(transpose_error, "cannot compute transposed layout of dynamic layout");
             }
             return transpose_layout_noexcept(l);
         }
@@ -80,7 +118,7 @@ namespace xt
         {
             if (sequence_size(permutation) != e.dimension())
             {
-                throw transpose_error("Permutation does not have the same size as shape");
+                XTENSOR_THROW(transpose_error, "Permutation does not have the same size as shape");
             }
 
             // permute stride and shape
@@ -97,7 +135,7 @@ namespace xt
             {
                 if (std::size_t(permutation[i]) >= e.dimension())
                 {
-                    throw transpose_error("Permutation contains wrong axis");
+                    XTENSOR_THROW(transpose_error, "Permutation contains wrong axis");
                 }
                 size_type perm = static_cast<size_type>(permutation[i]);
                 temp_shape[i] = e.shape()[perm];
@@ -128,7 +166,7 @@ namespace xt
                 {
                     if (permutation[i] == permutation[j])
                     {
-                        throw transpose_error("Permutation contains axis more than once");
+                        XTENSOR_THROW(transpose_error, "Permutation contains axis more than once");
                     }
                 }
             }
@@ -203,9 +241,9 @@ namespace xt
 #endif
     /// @endcond
 
-    /***************************
-     * ravel and flatten views *
-     ***************************/
+    /************************************
+     * ravel and flatten implementation *
+     ************************************/
 
     template <class I, class CI>
     class xiterator_adaptor;
@@ -244,6 +282,10 @@ namespace xt
         return ravel<L>(std::forward<E>(e));
     }
 
+    /*****************************
+     * trim_zeros implementation *
+     *****************************/
+
     /**
      * Trim zeros at beginning, end or both of 1D sequence.
      *
@@ -276,6 +318,10 @@ namespace xt
         return strided_view(std::forward<E>(e), { range(begin, end) });
     }
 
+    /**************************
+     * squeeze implementation *
+     **************************/
+
     /**
      * Returns a squeeze view of the given expression. No copy is made.
      * Squeezing an expression removes dimensions of extent 1.
@@ -327,11 +373,11 @@ namespace xt
             {
                 if (static_cast<std::size_t>(ix) > e.dimension())
                 {
-                    throw std::runtime_error("Axis argument to squeeze > dimension of expression");
+                    XTENSOR_THROW(std::runtime_error, "Axis argument to squeeze > dimension of expression");
                 }
                 if (e.shape()[static_cast<std::size_t>(ix)] != 1)
                 {
-                    throw std::runtime_error("Trying to squeeze axis != 1");
+                    XTENSOR_THROW(std::runtime_error, "Trying to squeeze axis != 1");
                 }
             }
             return squeeze_impl(std::forward<E>(e), std::forward<S>(axis), check_policy::none());
@@ -377,6 +423,10 @@ namespace xt
     }
     /// @endcond
 
+    /******************************
+     * expand_dims implementation *
+     ******************************/
+
     /**
      * @brief Expand the shape of an xexpression.
      *
@@ -388,13 +438,17 @@ namespace xt
      * @return returns a ``strided_view`` with expanded dimension
      */
     template <class E>
-    auto expand_dims(E&& e, std::size_t axis)
+    inline auto expand_dims(E&& e, std::size_t axis)
     {
         xstrided_slice_vector sv(e.dimension() + 1, all());
         sv[axis] = newaxis();
         return strided_view(std::forward<E>(e), std::move(sv));
     }
 
+    /*****************************
+     * atleast_Nd implementation *
+     *****************************/
+
     /**
      * Expand dimensions of xexpression to at least `N`
      *
@@ -408,7 +462,7 @@ namespace xt
      * @return ``strided_view`` with expanded dimensions
      */
     template <std::size_t N, class E>
-    auto atleast_Nd(E&& e)
+    inline auto atleast_Nd(E&& e)
     {
         xstrided_slice_vector sv((std::max)(e.dimension(), N), all());
         if (e.dimension() < N)
@@ -433,7 +487,7 @@ namespace xt
      * @sa atleast_Nd
      */
     template <class E>
-    auto atleast_1d(E&& e)
+    inline auto atleast_1d(E&& e)
     {
         return atleast_Nd<1>(std::forward<E>(e));
     }
@@ -443,7 +497,7 @@ namespace xt
      * @sa atleast_Nd
      */
     template <class E>
-    auto atleast_2d(E&& e)
+    inline auto atleast_2d(E&& e)
     {
         return atleast_Nd<2>(std::forward<E>(e));
     }
@@ -453,11 +507,15 @@ namespace xt
      * @sa atleast_Nd
      */
     template <class E>
-    auto atleast_3d(E&& e)
+    inline auto atleast_3d(E&& e)
     {
         return atleast_Nd<3>(std::forward<E>(e));
     }
 
+    /************************
+     * split implementation *
+     ************************/
+
     /**
      * @brief Split xexpression along axis into subexpressions
      *
@@ -471,11 +529,11 @@ namespace xt
      * @param axis axis along which to split the expression
      */
     template <class E>
-    auto split(E& e, std::size_t n, std::size_t axis = 0)
+    inline auto split(E& e, std::size_t n, std::size_t axis)
     {
         if (axis >= e.dimension())
         {
-            throw std::runtime_error("Split along axis > dimension.");
+            XTENSOR_THROW(std::runtime_error, "Split along axis > dimension.");
         }
 
         std::size_t ax_sz = e.shape()[axis];
@@ -485,7 +543,7 @@ namespace xt
 
         if (rest)
         {
-            throw std::runtime_error("Split does not result in equal division.");
+            XTENSOR_THROW(std::runtime_error, "Split does not result in equal division.");
         }
 
         std::vector<decltype(strided_view(e, sv))> result;
@@ -497,6 +555,38 @@ namespace xt
         return result;
     }
 
+    /**
+     * @brief Split an xexpression into subexpressions horizontally (column-wise)
+     *
+     * This method is equivalent to ``split(e, n, 1)``. 
+     *
+     * @param e input xexpression
+     * @param n number of elements to return
+     */
+    template <class E>
+    inline auto hsplit(E& e, std::size_t n)
+    {
+        return split(e, n, std::size_t(1));
+    }
+
+    /**
+     * @brief Split an xexpression into subexpressions vertically (row-wise)
+     *
+     * This method is equivalent to ``split(e, n, 0)``.
+     *
+     * @param e input xexpression
+     * @param n number of elements to return
+     */
+    template <class E>
+    inline auto vsplit(E& e, std::size_t n)
+    {
+        return split(e, n, std::size_t(0));
+    }
+
+    /***********************
+     * flip implementation *
+     ***********************/
+
     /**
      * @brief Reverse the order of elements in an xexpression along the given axis.
      * Note: A NumPy/Matlab style `flipud(arr)` is equivalent to `xt::flip(arr, 0)`,
@@ -527,6 +617,10 @@ namespace xt
         return strided_view(std::forward<E>(e), std::move(shape), std::move(strides), offset);
     }
 
+    /************************
+     * rot90 implementation *
+     ************************/
+
     template <std::ptrdiff_t N>
     struct rot90_impl;
 
@@ -592,14 +686,14 @@ namespace xt
      *
      * @return returns a view with the result of the rotation
      */
-    template <std::ptrdiff_t N = 1, class E>
-    inline auto rot90(E&& e, const std::array<std::ptrdiff_t, 2>& axes = {0, 1})
+    template <std::ptrdiff_t N, class E>
+    inline auto rot90(E&& e, const std::array<std::ptrdiff_t, 2>& axes)
     {
-        auto ndim = std::ptrdiff_t(e.shape().size());
+        auto ndim = static_cast<std::ptrdiff_t>(e.shape().size());
 
         if (axes[0] == axes[1] || std::abs(axes[0] - axes[1]) == ndim)
         {
-            throw std::runtime_error("Axes must be different");
+            XTENSOR_THROW(std::runtime_error, "Axes must be different");
         }
 
         auto norm_axes = forward_normalize<std::array<std::size_t, 2>>(e, axes);
@@ -607,6 +701,136 @@ namespace xt
 
         return rot90_impl<n>()(std::forward<E>(e), norm_axes);
     }
+
+    /***********************
+     * roll implementation *
+     ***********************/
+
+    /**
+     * @brief Roll an expression.
+     * The expression is flatten before shifting, after which the original
+     * shape is restore. Elements that roll beyond the last position are
+     * re-introduced at the first. This function does not change the input
+     * expression.
+     *
+     * @param e the input xexpression
+     * @param shift the number of places by which elements are shifted
+     * @param axis the axis along which elements are shifted.
+     *
+     * @return a roll of the input expression
+     */
+    template<class E>
+    inline auto roll(E&& e, std::ptrdiff_t shift)
+    {
+        auto cpy = empty_like(e);
+        auto flat_size = std::accumulate(cpy.shape().begin(), cpy.shape().end(), 1L, std::multiplies<std::size_t>());
+        while(shift < 0)
+        {
+            shift += flat_size;
+        }
+
+        shift %= flat_size;
+        std::copy(e.begin(), e.end() - shift,
+                  std::copy(e.end() - shift, e.end(), cpy.begin()));
+
+        return cpy;
+    }
+
+    namespace detail
+    {
+        /**
+         * Algorithm adapted from pythran/pythonic/numpy/roll.hpp
+         */
+
+        template < class To, class From, class S>
+        To roll(To to, From from, std::ptrdiff_t shift, std::size_t axis, S const& shape, std::size_t M)
+        {
+            std::ptrdiff_t dim = std::ptrdiff_t(shape[M]);
+            std::ptrdiff_t offset = std::accumulate(shape.begin() + M + 1, shape.end(), std::ptrdiff_t(1), std::multiplies<std::ptrdiff_t>());
+            if(shape.size() == M + 1)
+            {
+                if (axis == M)
+                {
+                    const auto split = from + (dim - shift) * offset;
+                    for(auto iter = split, end = from + dim * offset; iter != end; iter += offset, ++to)
+                    {
+                        *to = *iter;
+                    }
+                    for(auto iter = from, end = split; iter != end; iter += offset, ++to)
+                    {
+                        *to = *iter;
+                    }
+                }
+                else
+                {
+                    for(auto iter = from, end = from + dim * offset; iter != end; iter += offset, ++to)
+                    {
+                        *to = *iter;
+                    }
+                }
+            }
+            else
+            {
+                if (axis == M)
+                {
+                    const auto split = from + (dim - shift) * offset;
+                    for(auto iter = split, end = from + dim * offset; iter != end; iter += offset)
+                    {
+                        to = roll(to, iter, shift, axis, shape, M + 1);
+                    }
+                    for(auto iter = from, end = split; iter != end; iter += offset)
+                    {
+                        to = roll(to, iter, shift, axis, shape, M + 1);
+                    }
+                }
+                else
+                {
+                    for (auto iter = from, end = from + dim * offset; iter != end; iter += offset)
+                    {
+                        to = roll(to, iter, shift, axis, shape, M + 1);
+                    }
+                }
+            }
+            return to;
+        }
+    }
+
+    /**
+     * @brief Roll an expression along a given axis.
+     * Elements that roll beyond the last position are re-introduced at the first.
+     * This function does not change the input expression.
+     *
+     * @param e the input xexpression
+     * @param shift the number of places by which elements are shifted
+     * @param axis the axis along which elements are shifted.
+     *
+     * @return a roll of the input expression
+     */
+    template<class E>
+    inline auto roll(E&& e, std::ptrdiff_t shift, std::ptrdiff_t axis)
+    {
+        auto cpy = empty_like(e);
+        auto const& shape = cpy.shape();
+        std::size_t saxis = static_cast<std::size_t>(axis);
+        if(axis < 0)
+        {
+            axis += std::ptrdiff_t(cpy.dimension());
+        }
+
+        if(saxis >= cpy.dimension() || axis < 0)
+        {
+            XTENSOR_THROW(std::runtime_error, "axis is no within shape dimension.");
+        }
+
+        const auto axis_dim = static_cast<std::ptrdiff_t>(shape[saxis]);
+        while(shift < 0)
+        {
+            shift += axis_dim;
+        }
+
+        detail::roll(cpy.begin(), e.begin(), shift, saxis, shape, 0);
+        return cpy;
+    }
 }
 
 #endif
