@@ -19,6 +19,7 @@
 #include <type_traits>
 
 #include "xexception.hpp"
+#include "xtensor_config.hpp"
 #include "xtensor_simd.hpp"
 #include "xutils.hpp"
 
@@ -411,7 +412,7 @@ namespace xt
     inline auto uvector<T, A>::at(size_type i) -> reference
     {
         if(i >= size())
-            throw std::out_of_range("Out of range in uvector access");
+            XTENSOR_THROW(std::out_of_range, "Out of range in uvector access");
         return this->operator[](i);
     }
 
@@ -419,7 +420,7 @@ namespace xt
     inline auto uvector<T, A>::at(size_type i) const -> const_reference
     {
         if(i >= size())
-            throw std::out_of_range("Out of range in uvector access");
+            XTENSOR_THROW(std::out_of_range, "Out of range in uvector access");
         return this->operator[](i);
     }
 
@@ -899,7 +900,7 @@ namespace xt
     inline auto svector<T, N, A, Init>::at(size_type idx) -> reference
     {
         if(idx >= size())
-            throw std::out_of_range("Out of range in svector access");
+            XTENSOR_THROW(std::out_of_range, "Out of range in svector access");
         return this->operator[](idx);
     }
 
@@ -907,7 +908,7 @@ namespace xt
     inline auto svector<T, N, A, Init>::at(size_type idx) const -> const_reference
     {
         if(idx >= size())
-            throw std::out_of_range("Out of range in svector access");
+            XTENSOR_THROW(std::out_of_range, "Out of range in svector access");
         return this->operator[](idx);
     }
 
@@ -1581,12 +1582,20 @@ namespace xt
 #endif
         using value_type = std::size_t;
         using size_type = std::size_t;
+        using const_iterator = typename cast_type::const_iterator;
 
         constexpr static std::size_t size()
         {
             return sizeof...(X);
         }
 
+        template <std::size_t idx>
+        constexpr static auto get()
+        {
+            using cast_type = std::array<std::size_t, sizeof...(X)>;
+            return std::get<idx>(cast_type{X...});
+        }
+
         XTENSOR_FIXED_SHAPE_CONSTEXPR operator cast_type() const
         {
             return cast_type({X...});
